from oarepo_model_builder.builders import OutputBuilder
from oarepo_model_builder.outputs.cfg import CFGOutput


class WorkflowsSetupCfgBuilder(OutputBuilder):
    TYPE = "workflows_setup_cfg"

    def finish(self):
        super().finish()
        output: CFGOutput = self.builder.get_output("cfg", "setup.cfg")
        output.add_dependency("oarepo-workflows", ">=1.0.0")



from oarepo_model_builder.datatypes import DataTypeComponent, ModelDataType
from oarepo_model_builder.datatypes.components.model import (
    DefaultsModelComponent,
    PermissionsModelComponent,
    RecordMetadataModelComponent,
    RecordModelComponent,
    ServiceModelComponent,
)
from oarepo_model_builder.datatypes.components.model.utils import set_default
from oarepo_model_builder.utils.deepmerge import deepmerge
from oarepo_model_builder_drafts.datatypes.components import (
    DraftParentComponent,
    ParentMarshmallowComponent,
)


class WorkflowPermissionsComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    affects = [PermissionsModelComponent]

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if datatype.root.profile in {"record", "files", "draft_files"}:
            permissions = set_default(datatype, "permissions", {})
            permissions.setdefault("presets", ["workflow"])


class WorkflowComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    depends_on = [
        DefaultsModelComponent,
        RecordModelComponent,
        RecordMetadataModelComponent,
        ServiceModelComponent,
        ParentMarshmallowComponent,
        DraftParentComponent,
    ]

    def before_model_prepare(self, datatype, *, context, **kwargs):

        if datatype.root.profile == "record":
            datatype.definition["service-config"].setdefault("components", []).append(
                "{{oarepo_workflows.services.components.workflow.WorkflowComponent}}"
            )

            parent_fields = datatype.definition["draft-parent-record"].setdefault(
                "fields", {}
            )
            parent_fields.setdefault(
                "workflow",
                "{{oarepo_workflows.records.systemfields.workflow.WorkflowField}}()",
            )

            # add marshmallow-only field "state" to the parent schema
            deepmerge(
                datatype.definition["properties"].setdefault("state", {}),
                {
                    "type": "keyword",
                    "marshmallow": {"read": True, "write": False},
                    "ui": {"marshmallow": {"read": True, "write": False}},
                },
            )


class WorkflowOverrideDefaultsComponent(DataTypeComponent):
    eligible_datatypes = [ModelDataType]
    depends_on = [
        DefaultsModelComponent,
    ]
    affects = [
        RecordModelComponent,
        DraftParentComponent,
        ParentMarshmallowComponent,
    ]

    def before_model_prepare(self, datatype, *, context, **kwargs):
        if datatype.root.profile == "record":
            record = set_default(datatype, "record", {})
            fields = datatype.definition["record"].setdefault("fields", {})
            fields.setdefault(
                "state",
                "{{oarepo_workflows.records.systemfields.state.RecordStateField}}(initial='published')",
            )

            datatype.definition.setdefault(
                "draft-parent-record-metadata", {}
            ).setdefault(
                "base-classes",
                [
                    "oarepo_workflows.records.models.RecordWorkflowParentModelMixin",
                    "invenio_db.db{db.Model}",
                    "invenio_records.models.RecordMetadataBase",
                ],
            )

            datatype.definition.setdefault("parent-record-marshmallow", {}).setdefault(
                "base-classes",
                ["oarepo_workflows.services.records.schema.WorkflowParentSchema"],
            )

        if datatype.root.profile == "draft": # otherwise the default behavior is oarepo-model-builder-drafts
                                             # setting the default for before this
            record = set_default(datatype, "record", {})
            fields = datatype.definition["record"].setdefault("fields", {})
            fields.setdefault(
                "state",
                "{{oarepo_workflows.records.systemfields.state.RecordStateField}}()",
            )



from oarepo_model_builder.builders.json_base import JSONBaseBuilder
from oarepo_model_builder.outputs.yaml import YAMLOutput
from oarepo_model_builder.utils.dict import dict_get


class WorkflowSampleDataBuilder(JSONBaseBuilder):
    TYPE = "workflow_sample_data"
    output_file_type = "yaml"
    output_file_name = ["sample", "file"]
    parent_module_root_name = "jsonschemas"

    def begin(self, current_model, schema):
        output_name = dict_get(current_model.definition, self.output_file_name)
        path = self.builder.output_dir.joinpath(output_name)

        if path not in self.builder.outputs:
            # sample file does not exist, so skip json builder to not create an empty one
            super(JSONBaseBuilder, self).begin(current_model, schema)
            self.output = None
        else:
            super().begin(current_model, schema)

    def finish(self):
        output: YAMLOutput = self.output
        if output:
            # add parent: workflow: "default" to the sample data
            documents = output.documents
            for d in documents:
                d.setdefault("parent", {}).setdefault("workflow", "default")
            super().finish()



from .workflow import (
    WorkflowComponent,
    WorkflowOverrideDefaultsComponent,
    WorkflowPermissionsComponent,
)

WORKFLOW_COMPONENTS = [
    WorkflowComponent,
    WorkflowOverrideDefaultsComponent,
    WorkflowPermissionsComponent,
]
